@@ -3,14 +3,15 @@ from time import gmtime, strftime
 import numpy as np
 import logging
 from numpy import random
+import copy
 
-file_handler = logging.FileHandler(filename='nustar_utils_%s.log' % (strftime("%Y-%m-%dT%H:%M:%S", gmtime())))
-stdout_handler = logging.StreamHandler(sys.stdout)
-handlers = [stdout_handler, file_handler]
-
-logging.basicConfig(level=logging.INFO, format=' %(levelname)s - %(message)s', handlers=handlers)
+# file_handler = logging.FileHandler(filename='nustar_utils_%s.log' % (strftime("%Y-%m-%dT%H:%M:%S", gmtime())))
+# stdout_handler = logging.StreamHandler(sys.stdout)
+# handlers = [stdout_handler, file_handler]
+#
+# logging.basicConfig(level=logging.DEBUG, format=' %(levelname)s - %(message)s', handlers=handlers)
 #[%(asctime)s] {%(filename)s:%(lineno)d}
-logger = logging.getLogger()
+logger = logging.getLogger('')
 
 from subprocess import Popen, PIPE, STDOUT
 
@@ -193,6 +194,25 @@ n
 1000000000
 '''
 
+enphase_cmd_binfile='''17
+list_evt.txt
+none
+%s
+n
+%d
+
+%19.12e
+%19.12e
+0
+y
+%s
+0
+0
+0
+0
+1000000000
+'''
+
 enphase_cmd_orbit='''17
 list_evt.txt
 none
@@ -214,7 +234,43 @@ n
 1000000000
 '''
 
+enphase_cmd_orbit_binfile='''17
+list_evt.txt
+none
+%s
+y
+%s
+%d
+
+%19.12e
+%19.12e
+0
+y
+%s
+0
+0
+0
+0
+1000000000
+'''
+
+
+
+
 def make_enphase(freq,  min_en=3., max_en=70., en_step=0.5, n_bins=32, orbitfile=None, nudot=0):
+    '''
+    Wrapper on timingsuite to produce Energy - Phase matrixes for source and background for both FPM units.
+    :param freq: spin frequency
+    :param min_en: minimum energy (=3 keV
+    :param max_en: mximum energy (=70 keV)
+    :param en_step: energy step (=0.5 keV)
+    :param n_bins: number of phase bins (=32)
+    :param orbitfile: orbitfile for orbital correction (timingsuite) if None
+    :param nudot: spinn requency derivative (=0)
+    :return: it writes the fits files sourceA_ENPHASE.fits sourceB_ENPHASE.fits
+                        backgroundA_ENPHASE.fits backgroundB_ENPHASE.fits
+    '''
+
     for tt in ['source', 'background']:
         for unit in ['A', 'B']:
             with open('list_evt.txt', 'w') as ff:
@@ -222,19 +278,105 @@ def make_enphase(freq,  min_en=3., max_en=70., en_step=0.5, n_bins=32, orbitfile
 
             if orbitfile is None:
                 with open('timing_cmd.txt', 'w') as ff:
-                    ff.write(enphase_cmd % ('%s%s_ENPHASE.fits' % (tt, unit),
-                                     n_bins, freq, nudot, en_step, min_en, max_en) )
+                    if type(en_step) == float:
+                        ff.write(enphase_cmd % ('%s%s_ENPHASE.fits' % (tt, unit),
+                                         n_bins, freq, nudot, en_step, min_en, max_en) )
+                    else:
+                        ff.write(enphase_cmd_binfile % ('%s%s_ENPHASE.fits' % (tt, unit),
+                                                n_bins, freq, nudot, en_step))
             else:
                 if not os.path.isfile(orbitfile):
                     raise FileExistsError('File %s does not exist' % orbitfile)
 
                 with open('timing_cmd.txt', 'w') as ff:
-                    ff.write(enphase_cmd_orbit % ('%s%s_ENPHASE.fits' % (tt, unit), orbitfile,
+                    if type(en_step) == float:
+                        ff.write(enphase_cmd_orbit % ('%s%s_ENPHASE.fits' % (tt, unit), orbitfile,
                                      n_bins, freq, nudot, en_step, min_en, max_en) )
+                    else:
+                        ff.write(enphase_cmd_orbit_binfile % ('%s%s_ENPHASE.fits' % (tt, unit), orbitfile,
+                                                      n_bins, freq, nudot, en_step))
 
             run()
 
+def pad_matrices_with_zeros(x1_min, x1_max, pp1, dpp1, x2_min, x2_max, pp2, dpp2, tolerance=1e-2):
+    '''
+    :param x1_min:
+    :param x1_max:
+    :param pp1:
+    :param dpp1:
+    :param x2_min:
+    :param x2_max:
+    :param pp2:
+    :param dpp2:
+    :param tolerance:
+    :return:
+    '''
+    if len(x1_min) == len(x2_min):
+        diff = np.sum(x1_min - x2_min)
+        if np.abs(diff) > tolerance:
+            logger.warning("vector lower edges have same size but likely different values")
+        return x1_min, x1_max, pp1, dpp1, x2_min, x2_max, pp2, dpp2
+
+    if len(x1_min) < len(x2_min):
+        new_x1_min, new_x1_max, new_pp1, new_dpp1 = pad_matrix_with_zeros(x1_min, x1_max, pp1, dpp1, x2_min, x2_max)
+        return new_x1_min, new_x1_max, new_pp1, new_dpp1, x2_min, x2_max, pp2, dpp2
+
+    if len(x1_min) > len(x2_min):
+        new_x2_min, new_x2_max, new_pp2, new_dpp2 = pad_matrix_with_zeros(x2_min, x2_max, pp2, dpp2, x1_min, x1_max)
+        return x1_min, x1_max, pp1, dpp1, new_x2_min, new_x2_max, new_pp2, new_dpp2
+
+
+
+def pad_matrix_with_zeros(x1_min, x1_max, pp1, dpp1, x2_min, x2_max):
+    '''
+    returns matrix padded with zeros to fill the gaps
+    :param x1_min:
+    :param x1_max:
+    :param pp1:
+    :param dpp1:
+    :param x2_min:
+    :param x2_max:
+    :return:
+    '''
+
+    new_x1_min = []
+    new_x1_max = []
+    new_pp1 = []
+    new_dpp1 = []
+    n_bins = pp1.shape[1]
+    for y, z in zip(x2_min, x2_max):
+        found = False
+        for i in range(len(x1_min)):
+            if x1_min[i] == y:
+                new_x1_min.append(x1_min[i])
+                new_x1_max.append(x1_max[i])
+                new_pp1.append(pp1[i, :])
+                new_dpp1.append(dpp1[i, :])
+                found = True
+
+        if not found:
+            new_x1_min.append(y)
+            new_x1_max.append(z)
+            new_pp1.append(np.zeros(n_bins))
+            new_dpp1.append(np.zeros(n_bins))
+
+    if len(x2_min) != len(new_x1_min):
+        raise RuntimeError('PAdding matrix with zeros gave wrong sizes !!')
+
+    return np.array(new_x1_min), np.array(new_x1_max), np.array(new_pp1), np.array(new_dpp1)
+
+
 def read_one_matrix(kind, unit, background_difference_limit=5):
+    '''
+
+    :param kind: 'E' for Energy-Phase, 'T' for Time-Phase
+    :param unit: 'A' or 'B' for FPM unit
+    :param background_difference_limit: (if size differs by more than 5, it will skip background subtraction
+    :return: x_min lower edge of time or energy bins
+             x_min upper edge of time or energy bins,
+             pp matrix
+             dpp matrix of uncertainties
+    '''
     import astropy.io.fits as pf
 
     if not (unit == 'A' or unit == 'B'):
@@ -262,36 +404,53 @@ def read_one_matrix(kind, unit, background_difference_limit=5):
 
     ff = pf.open(fname_bck, 'readonly')
     t_minAb = ff[1].data[key1]
-    #t_maxAb = ff[1].data[key2]
+    t_maxAb = ff[1].data[key2]
     ptbA = ff[1].data['MATRIX']
     dptbA = ff[1].data['ERROR']
     ff.close()
 
-    if np.abs(len(t_minA) - len(t_minAb)) <  background_difference_limit:
-        indA, indAb = get_ind_combine(t_minA, t_minAb)
-        t_minA = t_minA[indA]
-        t_maxA = t_maxA[indA]
-
-        ptA = ptA[indA, :] - ptbA[indAb, :]
-        dptA = np.sqrt(dptA[indA, :] ** 2 + dptbA[indAb, :] ** 2)
-        logger.info('Subtracted the background')
+    if background_difference_limit <= 0:
+        t_minA, t_maxA, ptA, dptA, t_minAb, t_maxAb, ptbA, dptbA = \
+            pad_matrices_with_zeros(t_minA, t_maxA, ptA, dptA, t_minAb, t_maxAb, ptbA, dptbA)
+        ptA -= ptbA
+        dptA = np.sqrt(dptbA ** 2 + dptA ** 2)
     else:
-        logger.info('We do not subtract the background because the difference in matrix dimension is %d' %
-                    np.abs(len(t_minA) - len(t_minAb)))
+        if np.abs(len(t_minA) - len(t_minAb)) < background_difference_limit:
+            indA, indAb = get_ind_combine(t_minA, t_minAb)
+            t_minA = t_minA[indA]
+            t_maxA = t_maxA[indA]
+
+            ptA = ptA[indA, :] - ptbA[indAb, :]
+            dptA = np.sqrt(dptA[indA, :] ** 2 + dptbA[indAb, :] ** 2)
+            logger.info('Subtracted the background')
+        else:
+            logger.info('We do not subtract the background because the difference in matrix dimension is %d' %
+                        np.abs(len(t_minA) - len(t_minAb)))
 
     return t_minA, t_maxA, ptA, dptA
 
 def read_and_sum_matrixes(kind, background_difference_limit=5):
+    '''
+
+    :param kind: 'E' for ENERGY-Phase, 'T'  for 'TIME-Phase
+    :param background_difference_limit:
+    :return:
+    '''
     t_minA, t_maxA, ptA, dptA = read_one_matrix(kind, 'A', background_difference_limit=background_difference_limit)
     t_minB, t_maxB, ptB, dptB = read_one_matrix(kind, 'B', background_difference_limit=background_difference_limit)
 
-    indA, indB = get_ind_combine(t_minA, t_minB)
+    if background_difference_limit > 0 :
+        indA, indB = get_ind_combine(t_minA, t_minB)
+        t_min = t_minA[indA]
+        t_max = t_maxA[indA]
+        pp = ptA[indA, :] + ptB[indB, :]
+        dpp = np.sqrt(dptA[indA, :] ** 2 + dptB[indB, :] ** 2)
+    else:
+        t_min, t_max, ptA, dptA, t_minB, t_maxB, ptB, dptB = \
+            pad_matrices_with_zeros(t_minA, t_maxA, ptA, dptA, t_minB, t_maxB, ptB, dptB)
 
-    t_min = t_minA[indA]
-    t_max = t_maxA[indA]
-
-    pp = ptA[indA, :] + ptB[indB, :]
-    dpp = np.sqrt(dptA[indA, :] ** 2 + dptB[indB, :] ** 2)
+        pp = ptA + ptB
+        dpp = np.sqrt(dptA ** 2 + dptB ** 2)
 
     logger.info('Matrix for unit A has size %d x %d ' % (ptA.shape[0], ptA.shape[1]))
     logger.info('Matrix for unit B has size %d x %d ' % (ptB.shape[0], ptB.shape[1]))
@@ -300,6 +459,12 @@ def read_and_sum_matrixes(kind, background_difference_limit=5):
     return t_min, t_max, pp, dpp
 
 def get_ind_combine_engine(x1, x2):
+    '''
+    Util function to find index of common values in x1 and x2
+    :param x1:
+    :param x2:
+    :return:
+    '''
     ind1 = []
     ind2 = []
     n_removed = 0
@@ -319,6 +484,12 @@ def get_ind_combine_engine(x1, x2):
 
 
 def get_ind_combine(t_minA, t_minB):
+    '''
+    it calls get_ind_combine_engine after sorting for length
+    :param t_minA:
+    :param t_minB:
+    :return:
+    '''
     if len(t_minA) == len(t_minB):
         return np.arange(len(t_minA)), np.arange(len(t_minB))
     elif len(t_minA) < len(t_minB):
@@ -383,7 +554,7 @@ def make_tphase(freq,  min_en=3., max_en=70., t_step=1000, n_bins=32, orbitfile=
 
             run()
 
-def rebin_matrix(e_min, e_max, pp, dpp, min_s_n = 50, only_pulsed=False):
+def rebin_matrix(e_min, e_max, pp_input, dpp_input, min_s_n = 50, only_pulsed=False):
     '''
 
     :param e_min: array with minimum energy of each bin
@@ -404,35 +575,50 @@ def rebin_matrix(e_min, e_max, pp, dpp, min_s_n = 50, only_pulsed=False):
     new_e_mins = []
     new_e_maxs = []
     i1 = 0
-    while i1 < len(e_min) - 1:
-        if only_pulsed:
-            p1 = pp[i1, :] - np.mean(pp[i1, :])
-        else:
-            p1 = pp[i1, :]
+    rebinned_index = 0
 
-        dp1 = dpp[i1, :]
-        i1_store = i1
-        #print(i1)
-        logger.debug(i1)
+    pp = copy.deepcopy(pp_input)
+    dpp = copy.deepcopy(dpp_input)
+    while i1 < len(e_min) - 1:
+        p1 = copy.copy(pp[i1, :])
+        dp1 = copy.copy(dpp[i1, :])
+        logger.debug('%d' % i1)
         for i2 in range(i1 + 1, len(e_min)):
 
-            s_n = np.sum(np.abs(p1)) / np.sqrt(np.sum(dp1 ** 2))
+            ind = dp1 > 0
+            if only_pulsed:
+                s_n = np.sum(np.abs(p1[ind] - np.mean(p1[ind]))) / np.sqrt(np.sum(dp1[ind] ** 2))
+            else:
+                s_n = np.sum(np.abs(p1[ind])) / np.sqrt(np.sum(dp1[ind] ** 2))
+
             logger.debug(s_n)
             #print(np.sum(np.abs(p1)), np.sqrt(np.sum(dp1 ** 2)), s_n)
             if s_n >= min_s_n or i2 == len(e_min) - 1:
-                new_pulses.append(p1/float(i2-i1_store))
-                dnew_pulses.append(dp1/float(i2-i1_store))
-                new_e_mins.append(e_min[i1_store])
+                new_pulses.append(p1/float(i2-i1))
+                dnew_pulses.append(dp1/float(i2-i1))
+                new_e_mins.append(e_min[i1])
                 new_e_maxs.append(e_max[i2 - 1])
-                logger.debug("Boom", s_n, i1_store, i2)
-                #print("Boom", s_n, i1_store, i2)
+                logger.debug("Boom %f %d %d " % (s_n, i1, i2) )
+                # print("Boom", s_n, i1, i2)
+                # print(np.mean(p1)/float(i2-i1), np.mean(dp1)/float(i2-i1))
                 i1 = i2
+                logger.debug('Rebinned index : %d' % rebinned_index)
+                # print('Rebinned index : %d' % rebinned_index)
+                rebinned_index += 1
                 break
             else:
-                logger.debug("i2", i2)
+                logger.debug("i2 %d" % i2)
+
                 p1 += pp[i2, :]
                 dp1 = np.sqrt(dp1 ** 2 + dpp[i2, :] ** 2)
-                i1 = i2 - 1
+                # print("i2", i2)
+                # print(np.count_nonzero(p1), np.count_nonzero(pp[i2, :]))
+                #It may happen that 1 background photon (negative rate) makes a bin to zero
+                # if np.count_nonzero(p1) < old_nonzero:
+                #     print(p1)
+                #     print(pp[i2, :])
+                old_nonzero = np.count_nonzero(p1)
+
 
     logger.info('We rebinned from %d to %d bins at a minimum S/N of %.1f' % (len(e_min), len(new_e_mins), min_s_n))
     return np.array(new_e_mins), np.array(new_e_maxs), np.array(new_pulses), np.array(dnew_pulses)
@@ -499,7 +685,6 @@ def get_error_from_simul_rms(counts, counts_err, n_simul=100, n_harm=3):
 
 def get_error_from_simul(counts, counts_err, method, n_simul=100, use_poisson=False, **kwargs):
     '''
-
     :param counts: input data
     :param counts_err: input data error
     :param method: a fnction with the method to compute the quantity
@@ -565,7 +750,7 @@ def fft_pulsed_fraction(x, dx, level=0.1, n_harm_min=2, n_harm_max=-1, plot=Fals
         import matplotlib.pyplot as plt
         plt.figure()
         f = np.linspace(0, 1, n)
-        plt.errorbar(f, x, yerr=dx, marker='.', label='data')
+        plt.errorbar(f, x, yerr=dx, marker='.', label='data %s' % label)
         plt.plot(f, y, label='%d harmonics' % n_harm)
         plt.xlabel('Phase')
         plt.legend()
@@ -732,6 +917,16 @@ def plot_matrix_as_lines(t, pp, dpp, kind='T', normalize=False, offset=0):
     #plt.legend()
 
 def get_fourier_coeff(pulse):
+    '''
+    returns the first two harmonic coefficients
+    :param pulse:
+    :return:
+    phi0 phase of first harmonic
+    phi0_2 phase of second harmonic
+    a relative amplitude of first harmonic
+    a2 relative amplitude of second harmonic
+    '''
+
     phi = np.linspace(0, 2 * np.pi, len(pulse))
     i_c = np.sum(np.cos(phi) * pulse) / np.pi
     i_s = np.sum(np.sin(phi) * pulse) / np.pi
@@ -742,6 +937,7 @@ def get_fourier_coeff(pulse):
     phi0_2 = np.arctan2(i_s2, i_c2) / 2 / np.pi
     a = np.sqrt(i_c * i_c + i_s * i_s) / len(pulse) / np.mean(pulse)
     a2 = np.sqrt(i_c2 * i_c2 + i_s2 * i_s2) / len(pulse) / np.mean(pulse)
+
     return phi0, phi0_2, a, a2
 
 def get_fourier_coeff_error( counts, counts_err, n_simul=100):
